//
//  MenuViewController.m
//  MicroLifeDeviceSDK-Demo
//
//  Created by 吳憲有 on 2022/12/8.
//  Copyright © 2022 WiFi@MBP. All rights reserved.
//

#import "MenuViewController.h"

@interface MenuViewController ()

@end

@implementation MenuViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.menuTableView.backgroundColor = [UIColor whiteColor];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    [self BLESDK];
    [self loadData];
}

- (void)BLESDK {
    kWeakSelf(self)
    [self.sdk cancelAllConnect];
    self.bp3g = [MicroLife3GBP shareWhithAuthorizationkey:SDKkey_BPM];
    [self.bp3g setAutoConnect:NO];
    self.bp4g = [MicroLife4GBP shareWhithAuthorizationkey:SDKkey_BPM];
    [self.bp4g setAutoConnect:NO];
    self.temperature = [MicroLifeTemperature shareWhithAuthorizationkey:SDKkey_BT];
    [self.temperature setAutoConnect:NO];
    self.oxygen = [MicroLifeOxygen shareWhithAuthorizationkey:SDKkey_SPO2];
    [self.oxygen setAutoConnect:NO];
    self.weight = [MicroLifeWeight shareWhithAuthorizationkey:SDKkey_WEI];
    [self.weight setAutoConnect:NO];
    self.bloodSugar = [MicroLifeBloodSugar shareWhithAuthorizationkey:SDKkey_BS];
    [self.bloodSugar setAutoConnect:NO];
    self.bs = [MicroLifeBS shareWhithAuthorizationkey:SDKkey_BS];
    [self.bs setAutoConnect:NO];
    self.watchBPOffice = [MicroLifeWatchBPOffice shareWhithAuthorizationkey:SDKkey_WBP];
    [self.watchBPOffice setAutoConnect:NO];
    self.watchBPM1 = [MicroLifeWatchBPM1 shareWhithAuthorizationkey:SDKkey_WBP];
    [self.watchBPM1 setAutoConnect:NO];
    self.watchBPHome = [MicroLifeWatchBPHome shareWhithAuthorizationkey:SDKkey_WBP];
    [self.watchBPHome setAutoConnect:NO];
    self.watchBPO3 = [MicroLifeWatchBPO3 shareWhithAuthorizationkey:SDKkey_WBP];
    [self.watchBPO3 setAutoConnect:NO];
    self.peakFlowMeter = [MicroLifePFM shareWhithAuthorizationkey:SDKkey_PF];
    [self.peakFlowMeter setAutoConnect:NO];
    self.electrocardiography = [MicroLifeECG shareWhithAuthorizationkey:SDKkey_ECG];
    [self.electrocardiography setAutoConnect:NO];
    [self.sdk device:@[self.bp3g,self.bp4g,self.temperature,self.oxygen,self.weight,self.bloodSugar,self.bs,self.watchBPOffice,self.watchBPM1,self.watchBPHome,self.watchBPO3,self.peakFlowMeter,self.electrocardiography]];
    [self.sdk getDidUpdateStateBlock:^(CBManagerState state) {
        
    }];
    [self.sdk getScanDeviceBlock:^(id  _Nonnull device) {
        [weakself addDevice:device];
    } CancelScanBlock:^() {
        [weakself loadData];
    }];
}

- (void)addDevice:(MicroLifeDevice *)device {
    for (MenuInfo *m in self.menus) {
        if (m.deviceType == device.deviceType) {
            [m addDevice:device];
        }
    }
    kWeakSelf(self)
    dispatch_async(dispatch_get_main_queue(), ^{
        [weakself.menuTableView reloadData];
    });
}

- (void)loadData {
    for (MenuInfo *m in self.menus) {
        [m removalInfo];
    }
    kWeakSelf(self)
    dispatch_async(dispatch_get_main_queue(), ^{
        [weakself.menuTableView reloadData];
    });
    [self.sdk startScan];
}

#pragma mark -
- (NSMutableArray *)menus {
    if (!_menus) {
        self.menus = [MenuInfo menus];
    }
    return _menus;
}

#pragma mark - UITableViewDelegate,UITableViewDataSource
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.menus.count;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return 100;
}
    
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"Cell"];
    if (!cell) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:@"Cell"];
    }
    MenuInfo *info = [self.menus objectAtIndex:indexPath.row];
    if (info.name && info.UUID && info.RSSI) {
        cell.textLabel.text = [NSString stringWithFormat:@"%@\nName:%@ RSSI:%@\nUUID:%@ ",info.title,info.name,info.RSSI,info.UUID];
    } else {
        cell.textLabel.text = info.title;
    }
    cell.textLabel.numberOfLines = 3;
    cell.textLabel.textAlignment = NSTextAlignmentCenter;
    cell.textLabel.adjustsFontSizeToFitWidth = YES;
    cell.backgroundColor = info.backgroundColor;
    cell.textLabel.textColor = info.textColor;
    return cell;
}
    
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath{
    MenuInfo *info = [self.menus objectAtIndex:indexPath.row];
    switch (info.deviceType) {
        case 100:
            [self presentVC:@"OneViewController" Info:info];
            break;
        case 101:
            [self presentVC:@"CloudViewController" Info:info];
            break;
        case 102:
            if ([MFMailComposeViewController canSendMail]) {
                [self emailLog];
            } else {
                [self shareLog];
            }
            break;
        case DeviceType3GBP:
        case DeviceType4GBP:
            [self presentVC:@"BPMViewController" Info:info];
            break;
        case DeviceTypeWeight:
            [self presentVC:@"WEIViewController" Info:info];
            break;
        case DeviceTypeTemperature:
            [self presentVC:@"TEMPViewController" Info:info];
            break;
        case DeviceTypeOxygen:
            [self presentVC:@"OxygenViewController" Info:info];
            break;
        case DeviceTypeECG:
            [self presentVC:@"ECGViewController" Info:info];
            break;
        case DeviceTypeBloodSugar:
            [self presentVC:@"BSViewController" Info:info];
            break;
        case DeviceTypeWatchBPHome:
            [self presentVC:@"WBPViewController" Info:info];
            break;
        case DeviceTypeWatchBPO3:
            [self presentVC:@"WBO3ViewController" Info:info];
            break;
        case DeviceTypeWatchBPM1:
            [self presentVC:@"M1ViewController" Info:info];
            break;
        case DeviceTypeWatchBPOffice:
            [self presentVC:@"WBOViewController" Info:info];
            break;
        case DeviceTypePFM:
            [self presentVC:@"PFMViewController" Info:info];
            break;
        default:
            break;
    }
}

- (void)presentVC:(NSString *)identifier Info:(MenuInfo *)info {
    UIStoryboard *storyboard = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
    RootViewController *vc = [storyboard instantiateViewControllerWithIdentifier:identifier];
    vc.menuInfo = info;
    [self.navigationController pushViewController:vc animated:YES];
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
