//
//  RootViewController.m
//  MicroLifeDeviceSDK-Dome
//
//  Created by WiFi@ on 2018/5/21.
//  Copyright © 2018年 WiFi@MBP. All rights reserved.
//

#import "RootViewController.h"
#import "CloudViewController.h"

@interface RootViewController ()
@property (strong, nonatomic) NSString *logStr;

@property (weak, nonatomic) IBOutlet UIButton *cloudAPIBut;
@property (weak, nonatomic) IBOutlet UIButton *emailBut;

@property (nonatomic, copy  ) changedBlock   changedBlock;

@end

@implementation RootViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    [UIApplication sharedApplication].idleTimerDisabled = YES;

    // Do any additional setup after loading the view.
    NSString *appVersion = [[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleShortVersionString"];
    NSString *appBVersion = [[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleVersion"];
    self.VersionNumber.text = [NSString stringWithFormat:@"V %@(%@)",appVersion,appBVersion];
    
    self.MicroLifeDeviceSDKVersionNumber.text = [NSString stringWithFormat:@"DeviceSDKVersionNumber：%.1f",MicroLifeDeviceSDKVersionNumber];
    
    [self setNavigationTitle];
    
    self.logView.editable = NO;
    self.logView.layoutManager.allowsNonContiguousLayout = NO;
    
    kMicroLifeDeviceSDKShowLog(YES)
    
    // Bluetooth scanner establishment
    self.sdk = [BLESDK shareOne];
    
    [self test];
    
}

- (void)test {
    MicroLifeECGFunctionSettingValues *model = [MicroLifeECGFunctionSettingValues new];
    [model analysis:9 CMD:0x10 Data:@"16353E0128781700000000000000000000001E000000034D4C20454347000000000000000000000200" UUID:@"" Mac:[NSData new]];
    NSLog(@"test:%@",model.parseDictionary);
}

- (void)setNavigationTitle {
    self.navigationItem.title = self.menuInfo.title?:@"MicroLifeDeviceSDK";
    UIColor *backgroundColor = [IdeabusToolBox ib_RainbowColor:ColorDarkBlack Alpha:0.5];
    UIColor *titleColor = [IdeabusToolBox ib_ComplementaryColor:backgroundColor Alpha:1.0];
    //navigation標題文字顏色
    NSDictionary *dic = @{NSForegroundColorAttributeName:titleColor};
    if (@available(iOS 15.0, *)) {
        UINavigationBarAppearance *barApp = [UINavigationBarAppearance new];
        barApp.backgroundColor = backgroundColor;
        barApp.shadowColor = backgroundColor;
        barApp.titleTextAttributes = dic;
        self.navigationController.navigationBar.scrollEdgeAppearance = barApp;
        self.navigationController.navigationBar.standardAppearance = barApp;
    } else {
        //背景色
        self.navigationController.navigationBar.barTintColor = backgroundColor;
        self.navigationController.navigationBar.titleTextAttributes = dic;
        [self.navigationController.navigationBar setBackgroundImage:[UIImage new] forBarMetrics:UIBarMetricsDefault];
        [self.navigationController.navigationBar setShadowImage:[UIImage new]];
    }
    self.navigationController.navigationBar.tintColor = titleColor;
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    [self clearLogStr];
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    // Disconnect Device
    [self.sdk cancelAllConnect];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (MicroLifeUserInfo *)currUser {
    if (!_currUser) {
        MicroLifeUserInfo *user = [MicroLifeUserInfo new];
        user.bpmUserID = [self getBPMuserID];
        user.pfmUserID = [self getBPMuserID];
        user.weightUserID = [self getEBodyuserID];
        user.age = @([self getAge]);
        user.height = @(168);
        user.weight = @(70);
        user.gender = MicroLifeUserGenderMale;
        user.resistance = [self readNSUserDefaults:@"resistance"]?:@(0);
        user.roleType = MicroLifeUserRoleTypeNormal;
        _currUser = user;
    }
    return _currUser;
}

- (NSString *)logStr {
    if (!_logStr) {
        self.logStr = [NSString string];
    }
    return _logStr;
}

- (NSString *)date {
    return [IdeabusToolBox ib_CurrentDateWithFormat:@"yyyy-MM-dd HH:mm:ss:SSS"];
}

- (void)clearLogStr {
    self.logStr = @"";
    [self addLogWhitText:@"DEMO"];
    [self.logView scrollRangeToVisible:NSMakeRange(0, 0)];
}

- (void)addLogWhitText:(NSString *)text {
    dispatch_async(dispatch_get_main_queue(), ^{
        [self.view endEditing:true];
        NSLog(@"\n%@",text);
        NSString *logV = [NSString stringWithFormat:@"%@\n%@\n***\n",self.date,text];
        self.logStr = [self.logStr stringByAppendingFormat:@"%@",logV];
        self.logView.text = self.logStr;
        if (self.logStr.length > self.logView.frame.size.height) {
            [self.logView scrollRangeToVisible:NSMakeRange(0, self.logStr.length)];
        }
    });
}

- (NSString *)getBPMuserID {
    NSMutableString* BPMuserID = [[NSMutableString alloc] init];
    for (int i = 0; i < 11; i++) {
        if(i < 9){
            [BPMuserID appendString:[NSString stringWithFormat:@"%d", arc4random_uniform(10)]];
        } else {
            NSString *str = @"QWERTYUIOPASDFGHJKLZXCVBNM";
            [BPMuserID appendString:[NSString stringWithFormat:@"%@",[str substringWithRange:NSMakeRange(arc4random_uniform(26), 1)]]];
        }
    }
    return BPMuserID;
}

- (NSString *)getEBodyuserID {
    NSMutableString* EBodyuserID = [[NSMutableString alloc] init];
    for (int i = 0; i < 11; i++) {
        [EBodyuserID appendString:[NSString stringWithFormat:@"%d", arc4random_uniform(10)]];
    }
    return EBodyuserID;
}

- (NSInteger)getAge {
    return arc4random_uniform(81)+18;
}

#pragma mark - IBAction
- (IBAction)startSacn:(id)sender {
    if (self.currDeviceInfo != nil) {
        [self.currDeviceInfo disconnectDevice];
    } else {
        [self.sdk cancelAllConnect];
    }
    [self.sdk startScan];
    [self addLogWhitText:@"startScan"];
}

- (IBAction)cancelScan:(id)sender {
    [self.sdk cancelScan];
    [self addLogWhitText:@"cancelScan"];
}

- (IBAction)disconnectDevice:(id)sender {
    if (self.currDeviceInfo != nil) {
        [self.currDeviceInfo disconnectDevice];
        [self addLogWhitText:@"disconnectDevice"];
    } else {
        [self.sdk cancelAllConnect];
        [self addLogWhitText:@"cancelAllConnect"];
    }
}

- (IBAction)CloudAPIAction:(id)sender {
    
}

- (IBAction)EmailAction:(id)sender {
    [self emailLog];
}

- (IBAction)refeshAction:(id)sender {
    [self clearLogStr];
    [self.sdk cancelAllConnect];
    [self.sdk startScan];
}

- (IBAction)share:(id)sender {
    [self shareLog];
}

- (IBAction)email:(id)sender {
    if ([MFMailComposeViewController canSendMail]) {
        [self emailLog];
    } else {
        [self shareLog];
    }
}

- (void)shareLog {
    NSString *logPath = kMicroLifeDeviceSDKLogZip;
    NSURL *logURL = [NSURL fileURLWithPath:logPath];
    NSMutableArray *shares = [NSMutableArray arrayWithObject:logURL];
    if (self.filePath) {
        NSURL *fileURL = [NSURL fileURLWithPath:self.filePath];
        [shares addObject:fileURL];
    }
    if (self.rowPath) {
        NSURL *rowURL = [NSURL fileURLWithPath:self.rowPath];
        [shares addObject:rowURL];
    }
    [self shareWithActivityItems:shares CallBlock:^(BOOL completed) {
        
    }];
}

- (void)emailLog {
    kMicroLifeDeviceSDKRootVC(self)
    kMicroLifeDeviceSDKSendSupportMails(@"",[[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleName"], self.VersionNumber.text,@[@"willy.wu@microlife.com.tw"])
}

#pragma mark -
- (void)setupKeyboardShowHide {
    kWeakSelf(self)
    [self.view setupKeyboardShow:^(NSNotification * _Nonnull notifacation) {
        //such as
        NSDictionary* info = [notifacation userInfo];
        //get the size of key board
        CGSize kbSize = [[info objectForKey:UIKeyboardFrameEndUserInfoKey] CGRectValue].size;
        //make transform when key board show CGRect
        [weakself.view setFrame:CGRectMake(0, -kbSize.height, weakself.view.frame.size.width, weakself.view.frame.size.height)];
    } hide:^(NSNotification * _Nonnull notifacation) {
        [weakself.view setFrame:CGRectMake(0, 0, weakself.view.frame.size.width, weakself.view.frame.size.height)];
    }];
    [self.view setupForDismissKeyboard];
}

- (void)shareWithActivityItems:(NSArray *)activityItems CallBlock:(void (^)(BOOL completed))block {
    dispatch_async(dispatch_get_main_queue(), ^{
        UIActivityViewController *activityVC = [[UIActivityViewController alloc]initWithActivityItems:activityItems applicationActivities:nil];
        activityVC.completionWithItemsHandler = ^(NSString *activityType,BOOL completed,NSArray *returnedItems,NSError *activityError) {
            block(completed);
        };
        [self presentViewController:activityVC animated:YES completion:nil];
    });
}

#pragma mark - DataResponseDelegate
- (void)connectDeviceInfo:(id)device PeripheralState:(CBPeripheralState)state {
    NSString *log = [NSString stringWithFormat:@"connect Device：%@ PeripheralState：",device];
    switch (state) {
        case CBPeripheralStateConnecting:
            log = [log stringByAppendingFormat:@"Connecting"];
            break;
        case CBPeripheralStateConnected:
            self.currDeviceInfo = device;
            log = [log stringByAppendingFormat:@"Connected"];
            [self file:[device name]];
            break;
        case CBPeripheralStateDisconnecting:
            log = [log stringByAppendingFormat:@"Disconnecting"];
            break;
        case CBPeripheralStateDisconnected:
            self.currDeviceInfo = nil;
            log = [log stringByAppendingFormat:@"Disconnected"];
            break;
        default:
            break;
    }
    [self addLogWhitText:log];
}

- (void)log:(id)device CMD:(NSInteger)CMD Model:(id)model Error:(NSError *)error {
    if ([model isKindOfClass:[NSDictionary class]]) {
        NSString *log = [NSString stringWithFormat:@"%@ CMD：%02lX ReadData Dictionary：%@ error：%@", device, CMD, model, error];
        [self addLogWhitText:log];
        for (NSString *key in [model allKeys] ) {
            [self log:device CMD:CMD Model:model[key] Error:error];
        }
    } else if ([model isKindOfClass:[NSData class]]) {
        MicroLifeDataModel *dataModel = [MicroLifeDataModel stringWithValue:model];
        NSString *log = [NSString stringWithFormat:@"%@ CMD：%02lX ReadData Value：%@ error：%@", device, CMD, dataModel.dataValue, error];
        [self addLogWhitText:log];
        [self writeData:log];
    } else {
        NSString *log = [NSString stringWithFormat:@"%@ CMD：%02lX ReadData Model：%@ UUID：%@ error：%@", device, CMD, [model parseDictionary], [model UUID], error];
        [self addLogWhitText:log];
        [self writeData:log];
    }
}

- (void)file:(NSString *)name {
    NSArray *arry = NSSearchPathForDirectoriesInDomains(NSCachesDirectory,NSUserDomainMask, YES);

    self.dataLength = 0;
    
    NSString *date = [IdeabusToolBox ib_CurrentDateWithFormat:@"yyyy-MM-dd HH:mm"];
    
    self.fileName = [NSString stringWithFormat:@"%@[Log][%@].txt",name,date];
    self.filePath = [[arry lastObject]stringByAppendingPathComponent:self.fileName];
    
    self.rowName = [NSString stringWithFormat:@"%@[Row][%@].txt",name,date];
    self.rowPath = [[arry lastObject]stringByAppendingPathComponent:self.rowName];
}

- (void)writeData:(id)stringData {
    NSString* writeStr = [NSString stringWithFormat:@"%@ ",self.date];
    if ([stringData isKindOfClass:[NSData class]]) {
        NSData *data = stringData;
        writeStr = [writeStr stringByAppendingFormat:@"length = %ld, bytes = %@\n",data.length,[data.debugDescription uppercaseString]];
    } else {
        writeStr = [writeStr stringByAppendingFormat:@"%@\n",stringData];
    }
    NSData* writeData = [writeStr dataUsingEncoding:NSUTF8StringEncoding];
    NSFileManager *fileManager  = [NSFileManager defaultManager];
    BOOL createPathOk = YES;
    if (![fileManager fileExistsAtPath:[self.filePath stringByDeletingLastPathComponent] isDirectory:&createPathOk]) {
        [fileManager createDirectoryAtPath:[self.filePath stringByDeletingLastPathComponent] withIntermediateDirectories:YES attributes:nil error:nil];
    }
    if(![fileManager fileExistsAtPath:self.filePath]){
        [writeData writeToFile:self.filePath atomically:NO];
    }else{
        NSFileHandle* fileHandler = [NSFileHandle fileHandleForUpdatingAtPath:self.filePath];
        [fileHandler seekToEndOfFile];
        [fileHandler writeData:writeData];
        [fileHandler closeFile];
    }
}

- (void)writeRowData:(id)stringData {
    NSString* writeStr = @"";
    if ([stringData isKindOfClass:[NSData class]]) {
        NSData *data = stringData;
        writeStr = [writeStr stringByAppendingFormat:@"%@\n",[data.debugDescription uppercaseString]];
    } else {
        writeStr = [writeStr stringByAppendingFormat:@"%@\n",stringData];
    }
    NSData* writeData = [writeStr dataUsingEncoding:NSUTF8StringEncoding];
    NSFileManager *fileManager = [NSFileManager defaultManager];
    BOOL createPathOk = YES;
    if (![fileManager fileExistsAtPath:[self.rowPath stringByDeletingLastPathComponent] isDirectory:&createPathOk]) {
        [fileManager createDirectoryAtPath:[self.rowPath stringByDeletingLastPathComponent] withIntermediateDirectories:YES attributes:nil error:nil];
    }
    if (![fileManager fileExistsAtPath:self.rowPath]) {
        [writeData writeToFile:self.rowPath atomically:NO];
    } else {
        NSFileHandle *fileHandler = [NSFileHandle fileHandleForUpdatingAtPath:self.rowPath];
        [fileHandler seekToEndOfFile];
        [fileHandler writeData:writeData];
        [fileHandler closeFile];
    }
}

- (void)showHIinfPressurePickView:(NSString *)selectedItem ConfirmBlock:(confirmBlock)confirmBlock CancelBlock:(cancelBlock)cancelBlock HideBlock:(hideBlock)hideBlock {
    NSMutableArray *dataSource = [NSMutableArray arrayWithObjects:@"0",@"140",@"160",@"180",@"200",@"220",@"240",@"280", nil];
    IdeabusPickView *ideabusPickView = [[IdeabusPickView alloc] initWithDataSource:dataSource selectedItem:selectedItem titleName:@"HI_infPressure" confirmButtonName:NSLocalizedString(@"Save",nil) confirmBlock:confirmBlock cancelButtonName:NSLocalizedString(@"Cancel",nil) cancelBlock:cancelBlock hideBlock:hideBlock];
    [ideabusPickView show];
}

- (void)showIntervalPickView:(NSString *)selectedItem ConfirmBlock:(confirmBlock)confirmBlock CancelBlock:(cancelBlock)cancelBlock HideBlock:(hideBlock)hideBlock {
    NSMutableArray *dataSource = [NSMutableArray arrayWithObjects:@"5",@"10",@"15",@"20",@"30",@"60", nil];
    IdeabusPickView *ideabusPickView = [[IdeabusPickView alloc] initWithDataSource:dataSource selectedItem:selectedItem titleName:@"Interval" confirmButtonName:NSLocalizedString(@"Save",nil) confirmBlock:confirmBlock cancelButtonName:NSLocalizedString(@"Cancel",nil) cancelBlock:cancelBlock hideBlock:hideBlock];
    [ideabusPickView show];
}

#pragma mark - NSUserDefaults存取類
#pragma mark 將變量存至NSUserDefaults
- (void)saveNSUserDefaults:(id)value Key:(NSString *)key {
    [IdeabusToolBox ib_SaveNSUserDefaults:value Key:key];
}

#pragma mark 從NSUserDefaults讀取變量
- (id)readNSUserDefaults:(NSString *)key {
    return [IdeabusToolBox ib_ReadNSUserDefaults:key];
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
