//
//  WEIViewController.m
//  MicroLifeDeviceSDK-Dome
//
//  Created by WiFi@ on 2018/5/18.
//  Copyright © 2018年 WiFi@MBP. All rights reserved.
//

#import "WEIViewController.h"


@implementation WEIViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.ageTextField.text = self.currUser.age.stringValue;
    self.heightTextField.text = self.currUser.height.stringValue;
    self.weightTextField.text = self.currUser.weight.stringValue;
    self.genderSegmentedControl.selectedSegmentIndex = self.currUser.gender;
    kWeakSelf(self)
    [self connectWeight];
    [self.sdk device:@[self.weight]];
    [self.sdk getDidUpdateStateBlock:^(CBManagerState state) {
        NSString *log = [NSString stringWithFormat:@"DidUpdateState：%ld",(long)state];
        [weakself addLogWhitText:log];
    }];
    [self.sdk getScanDeviceBlock:^(id  _Nonnull device) {
        NSString *log = [NSString stringWithFormat:@"ScanDevice Name：%@ UUID：%@ mac：%@",[device name],[device UUID],[device mac]];
        [weakself addLogWhitText:log];
    } CancelScanBlock:^() {
        [weakself addLogWhitText:@"Cancel Scan"];
        [weakself.sdk startScan];
    }];
    [self.sdk getConnectDeviceStateBlock:^(id  _Nonnull device, CBCentralManager * _Nonnull central, CBPeripheral * _Nonnull peripheral, CBPeripheralState state, NSError * _Nonnull error) {
        [weakself connectDeviceInfo:device PeripheralState:state];
    } CancelAllDevicesConnectionBlock:^{
        [weakself addLogWhitText:@"Cancel All Devices Connection"];
        [weakself.sdk startScan];
    }];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    // Disconnect Device
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - IBAction
- (IBAction)comAction:(UIButton *)sender {
    switch (sender.tag) {
        case 100:
            [self writeUserData];
            break;
        case 101:
            [self.weight clearAllUsers];
            break;
        case 102:
            [self.weight readHistorys];
            break;
        case 103:
            [self.weight disconnect];
            break;
        default:
            break;
    }
}

- (void)writeUserData {
    [self.weight writeUserData:self.currUser.weightUserID Age:self.currUser.age.integerValue Gender:self.currUser.gender Height:self.currUser.height.integerValue RoleType:self.currUser.roleType Weight:self.currUser.weight.floatValue Resistance:self.currUser.resistance.integerValue];
}

- (IBAction)value:(id)sender {
    if ([sender isEqual:self.ageTextField]) {
        self.currUser.age = @(self.ageTextField.text.integerValue);
    }
    else if ([sender isEqual:self.heightTextField]) {
        self.currUser.height = @(self.heightTextField.text.integerValue);
    }
    else if ([sender isEqual:self.weightTextField]) {
        self.currUser.weight = @(self.weightTextField.text.floatValue);
    }
    else if ([sender isEqual:self.genderSegmentedControl]) {
        self.currUser.gender = self.genderSegmentedControl.selectedSegmentIndex;
    }
}

#pragma mark - device
- (void)connectWeight {
    kWeakSelf(self)
    self.weight = [MicroLifeWeight shareWhithAuthorizationkey:SDKkey_WEI];
    [self.weight deviceConnectedBlock:^{
        [weakself writeUserData];
    }];
    [self.weight getReadDataModelBlock:^(NSInteger CMD, id  _Nonnull model, NSError * _Nonnull error) {
        [weakself log:weakself.weight CMD:CMD Model:model Error:error];
        switch (CMD) {
            case CMDWeightWakeUpScale:
                [weakself addLogWhitText:@"WakeUp Scale"];
                [weakself writeUserData];
                [weakself.weight readHistorys];
                break;
            case CMDWeightSleepScale:
                [weakself addLogWhitText:@"Sleep Scale"];
                [weakself.weight disconnect];
                break;
            case CMDWeightMeasurementResult:
                [weakself addLogWhitText:@"Measurement Result"];
                weakself.currUser.resistance = ((MicroLifeBodyFat *)model).resistance;
                [weakself saveNSUserDefaults:weakself.currUser.resistance Key:@"resistance"];
                break;
            case CMDWeightReadHistorys:
                [weakself addLogWhitText:@"Read Historys"];
                weakself.currUser.resistance = ((MicroLifeBodyFat *)model).resistance;
                [weakself saveNSUserDefaults:weakself.currUser.resistance Key:@"resistance"];
                break;
            case CMDWeightUpdateUserInfo:
                [weakself addLogWhitText:@"Update UserInfo"];
                break;
            case CMDWeightClearAllUserInfo:
                [weakself addLogWhitText:@"Clear All UserInfo"];
                break;
            case CMDWeightNoMoreOfflineData:
                [weakself addLogWhitText:@"No More OfflineData"];
                break;
            case CMDWeightLowPower:
                [weakself addLogWhitText:@"Low Power"];
                break;
            case CMDWeightSyncSystemClock:
                [weakself addLogWhitText:@"Sync System Clock"];
                break;
            default:
                break;
        }
    }];
}
/*
 #pragma mark - Navigation
 
 // In a storyboard-based application, you will often want to do a little preparation before navigation
 - (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
 // Get the new view controller using [segue destinationViewController].
 // Pass the selected object to the new view controller.
 }
 */

@end
