//
//  MicroLifeBS.h
//  MicroLifeDeviceSDK
//
//  Created by 吳憲有 on 2023/2/9.
//

#import <MicroLifeDeviceSDK/MicroLifeDeviceSDK.h>

typedef NS_ENUM(NSInteger , CMDBS) {
    CMDBSQueryMachineFunction = 0x3000,
    CMDBSQueryBloodGlucoseDatas = 0xA100,
    CMDBSDownloadBloodGlucoseData = 0xA200,
    CMDBSQueryUricAcidDatas = 0xA300,
    CMDBSDownloadUricAcidData = 0xA400,
    CMDBSQueryCholesterolDatas = 0xA500,
    CMDBSDownloadCholesterolData = 0xA600,
    CMDBSQueryBloodKetoneDatas = 0xA700,
    CMDBSDownloadBloodKetoneData = 0xA800,
    CMDBSQueryGKIDatas = 0xA900,
    CMDBSDownloadGKIData = 0xAA00,
    CMDBSQueryLactateDatas = 0xAB00,
    CMDBSDownloadLactateData = 0xAC00,
    CMDBSQueryHemeDatas = 0xAD00,
    CMDBSDownloadHemeData = 0xAE00
};

/**
 PARAM

 - PARAMNone: 無
 - PARAMAll: 所有數據
 - PARAMLessSN1: 小於等於 SN1
 - PARAMMoreSN1: 大於等於 SN1
 - PARAMIncludeSN12: SN1到SN2之間 (包含)
 */
typedef NS_ENUM(NSInteger, PARAM) {
    PARAMNone = 0x0000,
    PARAMAll = 0x1100,
    PARAMLessSN1 = 0x1200,
    PARAMMoreSN1 = 0x1300,
    PARAMIncludeSN12 = 0x1400
};

NS_ASSUME_NONNULL_BEGIN

@interface MicroLifeBS : MicroLifeDevice

///  Query blood glucose machine function
- (void)QueryMachineFunction;

/// Query blood glucose data
/// - Parameters:
///   - PARAM: <#PARAM description#>
///   - SN1: <#SN1 description#>
///   - SN2: <#SN2 description#>
- (void)QueryBloodGlucoseDatas:(PARAM)PARAM SN1:(NSNumber *)SN1 SN2:(NSNumber *)SN2;

/// Download Blood Glucose Data
/// - Parameters:
///   - PARAM: <#PARAM description#>
///   - SN1: <#SN1 description#>
///   - SN2: <#SN2 description#>
- (void)DownloadBloodGlucoseData:(PARAM)PARAM SN1:(NSNumber *)SN1 SN2:(NSNumber *)SN2;

/// Query uric acid data
/// - Parameters:
///   - PARAM: <#PARAM description#>
///   - SN1: <#SN1 description#>
///   - SN2: <#SN2 description#>
- (void)QueryUricAcidDatas:(PARAM)PARAM SN1:(NSNumber *)SN1 SN2:(NSNumber *)SN2;

/// Download Uric Acid Data
/// - Parameters:
///   - PARAM: <#PARAM description#>
///   - SN1: <#SN1 description#>
///   - SN2: <#SN2 description#>
- (void)DownloadUricAcidData:(PARAM)PARAM SN1:(NSNumber *)SN1 SN2:(NSNumber *)SN2;

/// Query cholesterol data
/// - Parameters:
///   - PARAM: <#PARAM description#>
///   - SN1: <#SN1 description#>
///   - SN2: <#SN2 description#>
- (void)QueryCholesterolDatas:(PARAM)PARAM SN1:(NSNumber *)SN1 SN2:(NSNumber *)SN2;

/// Download Cholesterol Data
/// - Parameters:
///   - PARAM: <#PARAM description#>
///   - SN1: <#SN1 description#>
///   - SN2: <#SN2 description#>
- (void)DownloadCholesterolData:(PARAM)PARAM SN1:(NSNumber *)SN1 SN2:(NSNumber *)SN2;

/// Query blood ketone data
/// - Parameters:
///   - PARAM: <#PARAM description#>
///   - SN1: <#SN1 description#>
///   - SN2: <#SN2 description#>
- (void)QueryBloodKetoneDatas:(PARAM)PARAM SN1:(NSNumber *)SN1 SN2:(NSNumber *)SN2;

/// Download blood ketone data
/// - Parameters:
///   - PARAM: <#PARAM description#>
///   - SN1: <#SN1 description#>
///   - SN2: <#SN2 description#>
- (void)DownloadBloodKetoneData:(PARAM)PARAM SN1:(NSNumber *)SN1 SN2:(NSNumber *)SN2;

/// Query GKI data
/// - Parameters:
///   - PARAM: <#PARAM description#>
///   - SN1: <#SN1 description#>
///   - SN2: <#SN2 description#>
- (void)QueryGKIDatas:(PARAM)PARAM SN1:(NSNumber *)SN1 SN2:(NSNumber *)SN2;

/// Download GKI data
/// - Parameters:
///   - PARAM: <#PARAM description#>
///   - SN1: <#SN1 description#>
///   - SN2: <#SN2 description#>
- (void)DownloadGKIData:(PARAM)PARAM SN1:(NSNumber *)SN1 SN2:(NSNumber *)SN2;

/// Query Lactate Data
/// - Parameters:
///   - PARAM: <#PARAM description#>
///   - SN1: <#SN1 description#>
///   - SN2: <#SN2 description#>
- (void)QueryLactateDatas:(PARAM)PARAM SN1:(NSNumber *)SN1 SN2:(NSNumber *)SN2;

/// Download lactate data
/// - Parameters:
///   - PARAM: <#PARAM description#>
///   - SN1: <#SN1 description#>
///   - SN2: <#SN2 description#>
- (void)DownloadLactateData:(PARAM)PARAM SN1:(NSNumber *)SN1 SN2:(NSNumber *)SN2;

/// Query heme data
/// - Parameters:
///   - PARAM: <#PARAM description#>
///   - SN1: <#SN1 description#>
///   - SN2: <#SN2 description#>
- (void)QueryHemeDatas:(PARAM)PARAM SN1:(NSNumber *)SN1 SN2:(NSNumber *)SN2;

/// Download Heme Data
/// - Parameters:
///   - PARAM: <#PARAM description#>
///   - SN1: <#SN1 description#>
///   - SN2: <#SN2 description#>
- (void)DownloadHemeData:(PARAM)PARAM SN1:(NSNumber *)SN1 SN2:(NSNumber *)SN2;

@end

NS_ASSUME_NONNULL_END
