//
//  MicroLifeCloud2GManager.h
//  MicroLifeDeviceSDK
//
//  Created by willy.wu on 2022/5/20.
//

#import <MicroLifeDeviceSDK/MicroLifeDeviceSDK.h>
#import <MicroLifeDeviceSDK/MicroLifeCloudDataModel.h>

typedef void(^CloudDataBlock) (bool success, NSInteger code, MicroLifeCloudDataModel * _Nullable dataModel);
typedef void(^CloudDataFulfillBlock) (bool success, NSInteger code, bool fulfill, MicroLifeCloudDataModel * _Nullable dataModel);

NS_ASSUME_NONNULL_BEGIN

/**
 SeverType

 - MicroLifeTestSever: Test Sever
 - MicroLifeProductSever: Product Sever
 - MicroLifeTestUSASever: Test USA Sever
 - MicroLifeProductUSASever: Product USA Sever
 */
typedef NS_ENUM(NSInteger, MicroLifeSeverType) {
    MicroLifeDeveloperSever,
    MicroLifeTestSever,
    MicroLifeProductSever,
    MicroLifeTestUSASever,
    MicroLifeProductUSASever
};

/**
 CloudDataType
 - CloudDataTypeBPM:
 - CloudDataTypeWeight:
 - CloudDataTypeEvent:
 - CloudDataTypeTemperature:
 - CloudDataTypeOxygen:
 */
typedef NS_ENUM(NSInteger , MicroLifeCloudDataType) {
    CloudDataTypeBPM,
    CloudDataTypeWeight,
    CloudDataTypeEvent,
    CloudDataTypeTemperature,
    CloudDataTypeOxygen,
    CloudDataTypeEmail,
    CloudDataTypeDevice,
    CloudDataTypePushNotification,
    CloudDataTypeFAQ
};

/**
 CloudDataTimeType
 - CloudDataTimeModifyDate:
 - CloudDataTimeUpDate:
 */
typedef NS_ENUM(NSInteger , MicroLifeCloudDataTimeType) {
    CloudDataTimeModifyDate,
    CloudDataTimeUpDate
};

typedef NS_ENUM(NSInteger , MicroLifeContactSupportType) {
    ContactSupportTypeApp,
    ContactSupportTypeBloodpressure,
    ContactSupportTypeWeightFat,
    ContactSupportTypeTemperature,
    ContactSupportTypeOximeter
};

@interface MicroLifeCloud2GManager : CloudManager

/// <#Description#>
@property (nonatomic, assign) MicroLifeSeverType              severType;

/// <#Description#>
/// @param severType MicroLifeSeverType
///                  - MicroLifeTestSever: Test Sever
///                  - MicroLifeProductSever: Product Sever
///                  - MicroLifeProductUSASever: Product USA Sever
/// @param client_id <#client_id description#>
/// @param client_secret <#client_secret description#>
/// @param showLog <#showLog description#>
+ (instancetype)shareWhithSeverType:(MicroLifeSeverType)severType ID:(NSString *)client_id Secret:(NSString *)client_secret showLog:(BOOL)showLog;

/// <#Description#>
/// @param severType MicroLifeSeverType
///                  - MicroLifeTestSever: Test Sever
///                  - MicroLifeProductSever: Product Sever
///                  - MicroLifeProductUSASever: Product USA Sever
- (void)setMainSeverType:(MicroLifeSeverType)severType;

/// <#Description#>
/// @param countryCode <#countryCode description#>
- (NSString *)checkRegion:(NSString *)countryCode;

/// <#Description#>
/// @param countryCode <#countryCode description#>
- (void)setRegion:(NSString *)countryCode;

/// <#Description#>
/// @param currentLanguage <#currentLanguage description#>
- (NSString *)checkCurrentLanguage:(NSString *)currentLanguage;

/// <#Description#>
/// @param currentLanguage <#currentLanguage description#>
- (void)setCurrentLanguage:(NSString *)currentLanguage;

/// <#Description#>
/// @param showLog <#showLog description#>
- (void)showLog:(BOOL)showLog;

/// <#Description#>
- (NSString *)mainURL;

/// <#Description#>
/// @param vc <#vc description#>
/// @param redirect_url <#redirect_uri description#>
/// @param flag <#flag description#>
/// @param completion <#completion description#>
- (void)loginWithVC:(id)vc RedirectURL:(NSString *)redirect_url animated:(BOOL)flag completion:(void (^)(void))completion;

/// <#Description#>
/// @param vc <#vc description#>
/// @param redirect_url <#redirect_url description#>
/// @param flag <#flag description#>
/// @param completion <#completion description#>
- (void)logoutWithVC:(id)vc RedirectURL:(NSString *)redirect_url animated:(BOOL)flag completion:(void (^)(void))completion;

/// <#Description#>
/// @param vc <#vc description#>
/// @param flag <#flag description#>
/// @param completion <#completion description#>
- (void)changePasswordWithVC:(id)vc animated:(BOOL)flag completion:(void (^)(void))completion;

/// <#Description#>
/// @param grant_type <#grant_type description#>
/// @param code <#code description#>
/// @param redirect_uri redirect_uri description
/// @param block <#block description#>
- (void)postOAuthToken:(GrantType)grant_type code:(NSString *)code redirectURI:(NSString *)redirect_uri FinishBlock:(CloudDataBlock)block;

/// <#Description#>
/// @param client_unique_id <#client_unique_id description#>
/// @param os <#os description#>
/// @param machine_type <#machine_type description#>
/// @param push_unique_id <#push_unique_id description#>
/// @param model <#model description#>
/// @param company <#company description#>
/// @param gps <#gps description#>
/// @param block <#block description#>
- (void)postSys:(NSString *)client_unique_id os:(NSString *)os machine_type:(NSString *)machine_type push_unique_id:(NSString *)push_unique_id model:(NSString *)model company:(NSString *)company gps:(NSString *)gps FinishBlock:(CloudDataBlock)block;

/// <#Description#>
/// @param block <#block description#>
- (void)getMemberBaseData:(CloudDataBlock)block;

/// <#Description#>
/// @param block <#block description#>
- (void)getMemberData:(CloudDataBlock)block;

/// <#Description#>
/// @param name <#name description#>
/// @param birthday <#birthday description#>
/// @param gender <#gender description#>
/// @param height <#height description#>
/// @param weight <#weight description#>
/// @param unit_type <#unit_type description#>
/// @param sys_unit <#sys_unit description#>
/// @param sys <#sys description#>
/// @param dia <#dia description#>
/// @param goal_weight <#goal_weight description#>
/// @param body_fat <#body_fat description#>
/// @param bmi <#bmi description#>
/// @param sys_activity <#sys_activity description#>
/// @param dia_activity <#dia_activity description#>
/// @param weight_activity <#weight_activity description#>
/// @param body_fat_activity <#body_fat_activity description#>
/// @param bmi_activity <#bmi_activity description#>
/// @param threshold <#threshold description#>
/// @param cuff_size <#cuff_size description#>
/// @param bp_measurement_arm <#bp_measurement_arm description#>
/// @param date_format <#date_format description#>
/// @param conditions <#conditions description#>
/// @param photo <#photo description#>
/// @param block <#block description#>
- (void)postModifyMemberWhithName:(NSString *)name birthday:(NSString *)birthday gender:(NSInteger)gender height:(NSInteger)height weight:(float)weight unit_type:(NSInteger)unit_type sys_unit:(NSInteger)sys_unit sys:(float)sys dia:(float)dia goal_weight:(float)goal_weight body_fat:(float)body_fat bmi:(float)bmi sys_activity:(BOOL)sys_activity dia_activity:(BOOL)dia_activity weight_activity:(BOOL)weight_activity body_fat_activity:(BOOL)body_fat_activity bmi_activity:(BOOL)bmi_activity threshold:(NSInteger)threshold cuff_size:(NSInteger)cuff_size bp_measurement_arm:(NSInteger)bp_measurement_arm resistance:(NSInteger)resistance measuringTime:(NSInteger)measuringTime measuringTime_activity:(BOOL)measuringTime_activity oxygen:(NSInteger)oxygen oxygen_activity:(BOOL)oxygen_activity date_format:(NSInteger)date_format conditions:(NSString *)conditions photo:(UIImage *)photo FinishBlock:(CloudDataBlock)block;

/// <#Description#>
/// @param block <#block description#>
- (void)postDeletetModifyMember:(CloudDataBlock)block;

- (void)getHistoryDataWhithType:(MicroLifeCloudDataType)dataType Start:(NSString *)start End:(NSString *)end limit:(NSInteger)limit offset:(NSInteger)offset timeType:(MicroLifeCloudDataTimeType)time_type automatic:(BOOL)automatic FinishBlock:(CloudDataFulfillBlock)block;

/// <#Description#>
/// @param user_id <#user_id description#>
/// @param sys <#sys description#>
/// @param dia <#dia description#>
/// @param pul <#pul description#>
/// @param cuffokr <#cuffokr description#>
/// @param bpm_id <#bpm_id description#>
/// @param afib <#afib description#>
/// @param pad <#pad description#>
/// @param man <#man description#>
/// @param date <#date description#>
/// @param mac_address <#mac_address description#>
/// @param block <#block description#>
- (void)postAddBPMWhithID:(NSString *)user_id sys:(float)sys dia:(float)dia pul:(NSInteger)pul cuffokr:(NSInteger)cuffokr bpm_id:(NSString *)bpm_id afib:(NSInteger)afib pad:(NSInteger)pad man:(NSInteger)man date:(NSString *)date mac_address:(NSString *)mac_address FinishBlock:(CloudDataBlock)block;

/// <#Description#>
/// @param bpm_id <#bpm_id description#>
/// @param sys <#sys description#>
/// @param dia <#dia description#>
/// @param pul <#pul description#>
/// @param date <#date description#>
/// @param block <#block description#>
- (void)postEditBPMWhithID:(NSString *)bpm_id sys:(float)sys dia:(float)dia pul:(NSInteger)pul date:(NSString *)date FinishBlock:(CloudDataBlock)block;

/// <#Description#>
/// @param weight_id <#weight_id description#>
/// @param weight <#weight description#>
/// @param bmi <#bmi description#>
/// @param body_fat <#body_fat description#>
/// @param water <#water description#>
/// @param skeleton <#skeleton description#>
/// @param muscle <#muscle description#>
/// @param bmr <#bmr description#>
/// @param organ_fat <#organ_fat description#>
/// @param date <#date description#>
/// @param mac_address <#mac_address description#>
/// @param block <#block description#>
- (void)postAddWeightDataWhithID:(NSString *)weight_id weight:(float)weight bmi:(float)bmi body_fat:(float)body_fat water:(NSInteger)water skeleton:(float)skeleton muscle:(NSInteger)muscle bmr:(NSInteger)bmr organ_fat:(float)organ_fat date:(NSString *)date mac_address:(NSString *)mac_address FinishBlock:(CloudDataBlock)block;

/// <#Description#>
/// @param weight_id <#weight_id description#>
/// @param weight <#weight description#>
/// @param bmi <#bmi description#>
/// @param body_fat <#body_fat description#>
/// @param date <#date description#>
/// @param block <#block description#>
- (void)postEditWeightDataWhithID:(NSString *)weight_id weight:(float)weight bmi:(float)bmi body_fat:(float)body_fat date:(NSString *)date FinishBlock:(CloudDataBlock)block;

/// <#Description#>
/// @param event_code <#event_code description#>
/// @param event <#event description#>
/// @param type <#type description#>
/// @param event_time <#event_time description#>
/// @param block <#block description#>
- (void)postAddBTEventWhithCode:(NSInteger)event_code event:(NSString *)event type:(NSString *)type event_time:(NSString *)event_time FinishBlock:(CloudDataBlock)block;

/// <#Description#>
/// @param event_code <#event_code description#>
/// @param event <#event description#>
/// @param type <#type description#>
/// @param event_time <#event_time description#>
/// @param isDelete <#isDelete description#>
/// @param block <#block description#>
- (void)postEditBTEventWhithCode:(NSInteger)event_code event:(NSString *)event type:(NSString *)type event_time:(NSString *)event_time isDelete:(BOOL)isDelete FinishBlock:(CloudDataBlock)block;

/// <#Description#>
/// @param event_code <#event_code description#>
/// @param bt_id <#bt_id description#>
/// @param body_temp <#body_temp description#>
/// @param room_temp <#room_temp description#>
/// @param date <#date description#>
/// @param mac_address <#mac_address description#>
/// @param block <#block description#>
- (void)postAddBTDataWhithCode:(NSInteger)event_code bt_id:(NSString *)bt_id body_temp:(float)body_temp room_temp:(float)room_temp date:(NSString *)date mac_address:(NSString *)mac_address FinishBlock:(CloudDataBlock)block;

/// <#Description#>
/// @param event_code <#event_code description#>
/// @param bt_id <#bt_id description#>
/// @param body_temp <#body_temp description#>
/// @param room_temp <#room_temp description#>
/// @param date <#date description#>
/// @param block <#block description#>
- (void)postEditBTDataWhithCode:(NSInteger)event_code bt_id:(NSString *)bt_id body_temp:(float)body_temp room_temp:(float)room_temp date:(NSString *)date FinishBlock:(CloudDataBlock)block;

/// <#Description#>
/// @param oxymeter_id <#oxymeter_id description#>
/// @param spo2 <#spo2 description#>
/// @param pulse_rate <#pulse_rate description#>
/// @param perfusion_index <#perfusion_index description#>
/// @param spo2_raw_data <#spo2_raw_data description#>
/// @param pulse_rate_raw_data <#pulse_rate_raw_data description#>
/// @param perfusion_index_raw_data <#perfusion_index_raw_data description#>
/// @param spo2_waveform_data <#spo2_waveform_data description#>
/// @param measurement_length <#measurement_length description#>
/// @param date <#date description#>
/// @param mac_address <#mac_address description#>
/// @param block <#block description#>
- (void)postAddOxymeterDataWhithID:(NSString *)oxymeter_id spo2:(float)spo2 pulse_rate:(float)pulse_rate perfusion_index:(float)perfusion_index spo2_raw_data:(NSString *)spo2_raw_data pulse_rate_raw_data:(NSString *)pulse_rate_raw_data perfusion_index_raw_data:(NSString *)perfusion_index_raw_data spo2_waveform_data:(NSString *)spo2_waveform_data measurement_length:(NSInteger)measurement_length date:(NSString *)date mac_address:(NSString *)mac_address FinishBlock:(CloudDataBlock)block;

/// <#Description#>
/// @param oxymeter_id <#oxymeter_id description#>
/// @param spo2 <#spo2 description#>
/// @param pulse_rate <#pulse_rate description#>
/// @param perfusion_index <#perfusion_index description#>
/// @param measurement_length <#measurement_length description#>
/// @param date <#date description#>
/// @param block <#block description#>
- (void)postEditOxymeterDataWhithID:(NSString *)oxymeter_id spo2:(float)spo2 pulse_rate:(float)pulse_rate perfusion_index:(float)perfusion_index measurement_length:(NSInteger)measurement_length date:(NSString *)date FinishBlock:(CloudDataBlock)block;

/// <#Description#>
/// @param type_id <#type_id description#>
/// @param note_type <#note_type description#>
/// @param note <#note description#>
/// @param photo <#photo description#>
/// @param recording <#recording description#>
/// @param recording_time <#recording_time description#>
/// @param block <#block description#>
- (void)postAddNoteDataWhithID:(NSString *)type_id note_type:(NSInteger)note_type note:(NSString *)note photo:(UIImage *)photo recording:(NSData *)recording recording_time:(NSString *)recording_time FinishBlock:(CloudDataBlock)block;

/// <#Description#>
/// @param type_id <#type_id description#>
/// @param note_type <#note_type description#>
/// @param block <#block description#>
- (void)postDeleteRecordDataWhithID:(NSString *)type_id note_type:(NSString *)note_type FinishBlock:(CloudDataBlock)block;

/// <#Description#>
/// @param start_date <#start_date description#>
/// @param end_date <#end_date description#>
/// @param sys_threshold <#sys_threshold description#>
/// @param dia_threshold <#dia_threshold description#>
/// @param unit <#unit description#>
/// @param decimal <#decimal description#>
/// @param show_arrhythmia <#show_arrhythmia description#>
/// @param date_format <#date_format description#>
/// @param photo <#photo description#>
/// @param block <#block description#>
- (void)postDownloadBPMPDF:(NSString *)start_date end_date:(NSString *)end_date sys_threshold:(NSInteger)sys_threshold dia_threshold:(NSInteger)dia_threshold unit:(NSInteger)unit decimal:(NSInteger)decimal show_arrhythmia:(NSInteger)show_arrhythmia date_format:(NSInteger)date_format photo:(UIImage *)photo FinishBlock:(CloudDataBlock)block;

/// <#Description#>
/// @param start_date <#start_date description#>
/// @param end_date <#end_date description#>
/// @param event_code <#event_code description#>
/// @param threshold <#threshold description#>
/// @param unit <#unit description#>
/// @param decimal <#decimal description#>
/// @param date_format <#date_format description#>
/// @param photo <#photo description#>
/// @param block <#block description#>
- (void)postDownloadBTPDF:(NSString *)start_date end_date:(NSString *)end_date eventCode:(NSInteger)event_code threshold:(float)threshold unit:(NSInteger)unit decimal:(NSInteger)decimal date_format:(NSInteger)date_format photo:(UIImage *)photo FinishBlock:(CloudDataBlock)block;

/// <#Description#>
/// @param start_date <#start_date description#>
/// @param end_date <#end_date description#>
/// @param wei_threshold <#wei_threshold description#>
/// @param bmi_threshold <#bmi_threshold description#>
/// @param fat_threshold <#fat_threshold description#>
/// @param unit <#unit description#>
/// @param decimal <#decimal description#>
/// @param show_info <#show_info description#>
/// @param date_format <#date_format description#>
/// @param photo <#photo description#>
/// @param block <#block description#>
- (void)postDownloadWeightPDF:(NSString *)start_date end_date:(NSString *)end_date wei_threshold:(float)wei_threshold bmi_threshold:(float)bmi_threshold fat_threshold:(float)fat_threshold unit:(NSInteger)unit decimal:(NSInteger)decimal show_info:(NSInteger)show_info date_format:(NSInteger)date_format photo:(UIImage *)photo FinishBlock:(CloudDataBlock)block;

/// <#Description#>
/// @param start_date <#start_date description#>
/// @param end_date <#end_date description#>
/// @param threshold <#threshold description#>
/// @param unit <#unit description#>
/// @param decimal <#decimal description#>
/// @param date_format <#date_format description#>
/// @param photo <#photo description#>
/// @param block <#block description#>
- (void)postDownloadOxymeterPDF:(NSString *)start_date end_date:(NSString *)end_date threshold:(float)threshold unit:(NSInteger)unit decimal:(NSInteger)decimal date_format:(NSInteger)date_format photo:(UIImage *)photo FinishBlock:(CloudDataBlock)block;

/// <#Description#>
/// @param mac_address <#mac_address description#>
/// @param device_type <#device_type description#>
/// @param device_model <#device_model description#>
/// @param error_code <#error_code description#>
/// @param block <#block description#>
- (void)postAddDevice:(NSString *)mac_address device_type:(NSInteger)device_type device_model:(NSString *)device_model error_code:(NSString *)error_code FinishBlock:(CloudDataBlock)block;

/// <#Description#>
/// @param mac_address <#mac_address description#>
/// @param block <#block description#>
- (void)postDeleteDeviceData:(NSString *)mac_address FinishBlock:(CloudDataBlock)block;

/// <#Description#>
/// @param name <#name description#>
/// @param email <#email description#>
/// @param block <#block description#>
- (void)postAddMailNotification:(NSString *)name email:(NSString *)email FinishBlock:(CloudDataBlock)block;

/// <#Description#>
/// @param mail_id <#mail_id description#>
/// @param name <#name description#>
/// @param email <#email description#>
/// @param isDelete <#isDelete description#>
/// @param block <#block description#>
- (void)postEditMailNotification:(NSMutableArray<NSNumber *> *)mail_id name:(NSString *)name email:(NSString *)email isDelete:(BOOL)isDelete FinishBlock:(CloudDataBlock)block;

/// <#Description#>
/// @param csv <#csv description#>
/// @param block <#block description#>
- (void)uploadCSV:(NSData *)csv FinishBlock:(CloudDataBlock)block;

/// <#Description#>
/// @param import_id <#import_id description#>
/// @param block <#block description#>
- (void)inspectionSyncStatus:(NSString *)import_id FinishBlock:(CloudDataBlock)block;

/// <#Description#>
/// @param contact_name <#contact_name description#>
/// @param contact_email <#contact_email description#>
/// @param type <#type description#>
/// @param description <#description description#>
/// @param app_name <#app_name description#>
/// @param app_version <#app_version description#>
/// @param device_model <#device_model description#>
/// @param device_os_version <#device_os_version description#>
/// @param file <#file description#>
/// @param block <#block description#>
- (void)contactSupport:(NSString *)contact_name Email:(NSString *)contact_email Type:(MicroLifeContactSupportType)type Description:(NSString *)description AppName:(NSString *)app_name AppVersion:(NSString *)app_version DeviceModel:(NSString *)device_model DeviceOSVersion:(NSString *)device_os_version File:(id)file FinishBlock:(CloudDataBlock)block;

@end

NS_ASSUME_NONNULL_END
