//
//  MicroLifeWatchBPCurrentAndMData.h
//  MicroLifeDeviceSDK
//
//  Created by willy.wu on 2021/10/5.
// 

#import <MicroLifeDeviceSDK/MicroLifeDeviceSDK.h>

NS_ASSUME_NONNULL_BEGIN

@interface MicroLifeWatchBPCurrentAndMData : MicroLifeDataModel
/**
 Systolic
 */
@property (nonatomic, strong) NSNumber *sys;

/**
 Diastolic
 */
@property (nonatomic, strong) NSNumber *dia;

/**
 Pulse
 */
@property (nonatomic, strong) NSNumber *hr;

/**
 Year
 */
@property (nonatomic, strong) NSNumber *year;

/**
 Month
 */
@property (nonatomic, strong) NSNumber *month;

/**
 Date
 */
@property (nonatomic, strong) NSNumber *day;

/**
 Hour
 */
@property (nonatomic, strong) NSNumber *hour;

/**
 Minte
 */
@property (nonatomic, strong) NSNumber *minute;

/**
 the data detect with PAD  or Afib
 */
@property (nonatomic, strong) NSNumber *arr;

/**
 IHB
 */
@property (nonatomic, strong) NSNumber *IHB;

/**
 AFIb (If both detect Afib and IHB, only display Afib)
 */
@property (nonatomic, strong) NSNumber *AFIb;

/**
 the data is measured in usual mode
 */
@property (nonatomic, strong) NSNumber *usual;

/**
 the data is measured in diagnostic mode
 */
@property (nonatomic, strong) NSNumber *diagnostic;

/**
 the data is measured in night with diagnostic mode (ex. 4:00~12:00)
 */
@property (nonatomic, strong) NSNumber *AM;

/**
 the data is measured in morning with diagnostic mode (ex. 18:00~24:00)
 */
@property (nonatomic, strong) NSNumber *PM;

@property (nonatomic, strong) NSNumber *ABPM;
/**
 low battery
 */
@property (nonatomic, strong) NSNumber *LB;
/**
 anti-artifact detected
 */
@property (nonatomic, strong) NSNumber *AA;
/**
 start of a manual measurement
 */
@property (nonatomic, strong) NSNumber *SM;

//WatchBPO3 & WatchBPOffice
/**
 Condition
 0     BP
 1     BP +CBP
 2     Empty battery
 3     Error code for original BP
 4     stop measurement by manual
 5     Pill memo
 6     device switch off
 7     re-start during a 24h-profile
 10    Retry 1st -BP
 11    Retry 1st -BP+CBP
 12    Retry 1st -Empty battery
 13    Retry 1st -Error code for original BP
 14    Retry 1st -stop measurement by manual
 20    Retry 2nd -BP
 21    Retry 2nd -BP+CBP
 22    Retry 2nd -Empty battery
 23    Retry 2nd - Error code for original BP
 24    Retry 2nd - stop measurement by manual
 */
@property (nonatomic, strong) NSNumber *condition;

@property (nonatomic, strong) NSNumber *MAP;

@property (nonatomic, strong) NSNumber *MCBP;

@property (nonatomic, strong) NSNumber *PVR;

@property (nonatomic, strong) NSNumber *CSBP;

@property (nonatomic, strong) NSNumber *CPP;

/**
 Result Code    Error Code     Error Message    Description
 0x01           0x01           Error 1          Signals is too weak
 0x02           0x02           Error 2          Error signal
 0x03           0x03           Error 3          No pressure in the cuff
 0x05           0x05           Error 5          Abnormal result
 0x11           0x11           Error 11         Signal too weak during central blood pressure measuremen
 
 0x12           0x12           Error 12         Error signal during central blood pressure measurement
 
 0x13           0x13           Error 13         Cuff pressure errors during  central blood pressure measurement
 
 0x15           0x15           Error 15         Abnormal result of central blood pressure reading
 
 0x41           0x41           Error A          Flash memory error
 0x42                          Error B          Low battery
 0x46           0x46           Error F          The device has gone into “single fault condition”
 
 0x50           0x50           Error P          Manual stop measurement.
 0x51           0x51           Error A / Q      Failed to initialize system / Busy
 0x52           0x52           Error R          Past time
 */
@property (nonatomic, strong) NSNumber *errorCode;
@property (nonatomic, strong) NSNumber *resultCode;
@property (nonatomic, strong) NSString *errorMessage;

@end

NS_ASSUME_NONNULL_END
