//
//  MicroLifeWatchBPM1.h
//  MicroLifeDeviceSDK
//
//  Created by willy.wu on 2022/2/24.
//

#import <MicroLifeDeviceSDK/MicroLifeDeviceSDK.h>

typedef NS_ENUM(NSInteger , CMDWatchBPM1) {
    CMDWatchBPM1ReadHistorys = 0x00,
    CMDWatchBPM1ClearAllHistorys = 0x03,
    CMDWatchBPM1Disconnect = 0x04,
    CMDWatchBPM1ReadUserAndVersionData = 0x05,
    CMDWatchBPM1WriteUser = 0x06,
    CMDWatchBPM1ReadSettingValues = 0x07,
    CMDWatchBPM1WriteSettingValues = 0x08,
    CMDWatchBPM1ReadDeviceInfo = 0x0B,
    CMDWatchBPM1ReadDeviceTime = 0x0C,
    CMDWatchBPM1WriteDeviceTime = 0x0D,
    CMDWatchBPM1ReadSerialNumber = 0x0F,
    CMDWatchBPM1ReadFunctionSettingValue = 0x10,
    CMDWatchBPM1ReadBTModuleName = 0x13
};

NS_ASSUME_NONNULL_BEGIN

@interface MicroLifeWatchBPM1 : MicroLifeDevice

/**
 Read all history or current data from bpm
 */
- (void)readAllHistorys;

/**
 Clear all history data of the BPM
 */
- (void)clearAllHistorys;

/**
 Disconnect the Bluetooth with BPM
 */
- (void)disconnect;

/**
 Read user ID and version data from BPM
 */
- (void)readUserAndVersionData;

/**
 Write a new user ID to BPM
 
 @param ID user ID
 */
- (void)writeUserID:(NSString *)ID;

/**
 Read BPM setting values from BPM
 */
- (void)readSettingValues;

/// Write BPM setting values to BPM.
/// @param AUS_HI_infPressure Highest inflation pressure of AUS mode.
/// @param HI_infPressure Highest inflation pressure of Auto mode.
/// @param SW_AUTO_hide Set Show readings during rest time in auto mode. true:hide/false:show.
/// @param SW_SEL_silent Beeper true:enabled/false:disabled
/// @param SW_AUS_Hide Set Show cuff pressure during deflation in AUS mode. true:hide/false:show.
/// @param SW_AVG_no_include_first Set Average is include first memory data. true:is/false:is not.
/// @param SW_CBP Set CBP measurement  true:enabled/false:disabled.
/// @param SW_AFib Set AFib measurement  true:enabled/false:disabled.
/// @param SW_AMPM Set 12/24-hour clock true:12-hour/false:24-hour.
/// @param SW_Kpa Set Pressure unit: true:Kpa/false:mmHg.
/// @param RestTime Rest time of auto mode. Start countdown base on rest time before 1st measurement in auto mode.
/// @param IntervalTime Interval time of auto mode. Start countdown base on interval time before 2nd~6th measurement in auto mode.
/// @param AutoMeasureNumber It’s number of measurements in auto mode.

- (void)writeSettingValuesWithSelectedAUS_HI_infPressure:(HIinfPressure)AUS_HI_infPressure HI_infPressure:(HIinfPressure)HI_infPressure SW_AUTO_hide:(BOOL)SW_AUTO_hide SW_SEL_silent:(BOOL)SW_SEL_silent SW_AUS_Hide:(BOOL)SW_AUS_Hide SW_AVG_no_include_first:(BOOL)SW_AVG_no_include_first SW_CBP:(BOOL)SW_CBP SW_AFib:(BOOL)SW_AFib SW_AMPM:(BOOL)SW_AMPM SW_Kpa:(BOOL)SW_Kpa RestTime:(NSInteger)RestTime IntervalTime:(NSInteger)IntervalTime AutoMeasureNumber:(NSInteger)AutoMeasureNumber;

/**
 Read device ID and info from BPM
 */
- (void)readDeviceIDAndInfo;

/**
 Read device Time from BPM
 */
- (void)readDeviceTime;

/**
 Write device Time to BPM
 */
- (void)writeDeviceTime;

///Read device SN from BPM
- (void)readSerialNumber;

/**
 Read BPM function setting value from BPM
 */
- (void)readFunctionSettingValue;

/**
 Read BT module name from BPM
 */
- (void)readBTModuleName;

#pragma <<Only for factory use>>
- (void)writeDeviceID:(NSString *)ID;
- (void)writeSerialNumber:(NSString *)SN;
- (void)writeFunctionSettingValueAFib:(BOOL)Fun_SEL_AFib AMPM:(BOOL)Fun_SEL_AMPM AutoTest:(BOOL)Fun_SEL_AutoTest Bluetooth:(BOOL)Fun_SEL_Bluetooth UnitKpa:(BOOL)Fun_SEL_UnitKpa RS232:(BOOL)Fun_SEL_RS232;
- (void)writeBTModuleName:(NSString *)name;

@end

NS_ASSUME_NONNULL_END
